------------------------------------------------- Globals -------------------------------------------------
local size = 64
local titlebarSize = 16
local selection
local textures = {}
local shown = false
local glyphCursor = 1
local rowCursor = 1
local addonMessagePrefix = "GGH"

local function CreateFrameWrapper(type, name, parent, inheritsFrame)
    local _, _, _, wowtocversion = GetBuildInfo()
    if inheritsFrame == nil then
        inheritsFrame = "BackdropTemplate"
    end

    if wowtocversion > 90000 then
        return CreateFrame(type, name, parent, inheritsFrame)
    end

    return CreateFrame(type, name, parent, inheritsFrame)
end

local function defaultBackdrop(path)
    return {
        bgFile = path,
        edgeFile = "",
        tile = true,
        tileSize = 32,
        edgeSize = 1,
        insets = {
            left = 0,
            right = 0,
            top = 0,
            bottom = 0
        }
    }
end

local function resetSelection()
    selection = {}
    for i = 1, 4 do
        selection[i] = {}
        for j = 1, 3 do
            selection[i][j] = {
                sel = nil
            }
        end
    end
end

local function countRowSelection(row, state)
    local count = 0
    for i = 1, 4 do
        if selection[i][row].sel == state then
            count = count + 1
        end
    end
    return count
end

local function isRowSelected(row)
    for i = 1, 4 do
        if selection[i][row].sel == nil then
            return false
        end
    end
    return true
end

local function createBottomBorder(frame)
    local l = frame:CreateLine(nil, "OVERLAY")
    l:SetColorTexture(.52, .52, .52, 1)
    l:SetStartPoint("BOTTOMLEFT", 0, 0)
    l:SetEndPoint("BOTTOMRIGHT", 0, 0)
    l:SetThickness(2)
    return l
end

local function createSideBorder(frame)
    local l = frame:CreateLine(nil, "OVERLAY")
    l:SetColorTexture(.52, .52, .52, 1)
    l:SetStartPoint("TOPRIGHT", 0, 0)
    l:SetEndPoint("BOTTOMRIGHT", 0, 0)
    l:SetThickness(2)
    return l
end

local function createTopBorder(frame)
    local l = frame:CreateLine(nil, "OVERLAY")
    l:SetColorTexture(.52, .52, .52, 1)
    l:SetStartPoint("TOPLEFT", 0, 0)
    l:SetEndPoint("TOPRIGHT", 0, 0)
    l:SetThickness(2)
    return l
end

local function frameSetTexture(frame, path, layer)
    layer = layer or "MEDIUM"
    local texture = frame:CreateTexture(nil, layer)
    texture:SetTexture(path)
    texture:SetAllPoints(frame)
    frame:SetNormalTexture(texture)
end

local function frameSetTextureWithDim(frame, path, layer, width, height)
    layer = layer or "MEDIUM"
    local texture = frame:CreateTexture(nil, layer)
    texture:SetTexture(path)
    texture:SetPoint("CENTER", frame, "CENTER")
    texture:SetWidth(width)
    texture:SetHeight(height)
    frame:SetNormalTexture(texture)
end

textures["leaf_grey_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\leaf_grey_128"
textures["leaf_white_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\leaf_white_128"
textures["leaf_fill_grey_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\leaf_fill_grey_128"
textures["leaf_fill_white_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\leaf_fill_white_128"
textures["flower_grey_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\flower_grey_32"
textures["flower_white_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\flower_white_32"
textures["flower_white_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\flower_white_128"
textures["flower_fill_grey_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\flower_fill_grey_128"
textures["flower_fill_white_128"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\flower_fill_white_128"
textures["circle_grey_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\circle_grey_32"
textures["circle_white_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\circle_white_32"
textures["cross_grey_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\cross_grey_32"
textures["cross_white_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\cross_white_32"
textures["refresh_32"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\refresh_32"
textures["refresh_256"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\refresh_256"
textures["circle2"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\circle2"
textures["main_backdrop"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\main_background"
textures["red_backdrop"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\red_background"
textures["hover_backdrop"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\hover_background"
textures["edge"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\edge"
textures["marker"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\marker"
textures["box_hollow_grey"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\box_hollow_grey"
textures["box_hollow_white"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\box_hollow_white"
textures["box_fill_grey"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\box_fill_grey"
textures["box_fill_white"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\box_fill_white"
textures["undef"] = "Interface\\AddOns\\GuessingGameHelper\\texture\\undef"
resetSelection()

------------------------------------------------- Main Frame -------------------------------------------------
local mainFrame = CreateFrameWrapper("Frame", "GGH_Main", UIParent)
mainFrame:SetFrameStrata("BACKGROUND")
mainFrame:SetWidth(size * 4)
mainFrame:SetHeight(size + titlebarSize)
mainFrame:SetMovable(true)
mainFrame:EnableMouse(true)
mainFrame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))

mainFrame:RegisterForDrag("LeftButton")
mainFrame:SetScript("OnDragStart", mainFrame.StartMoving)
mainFrame:SetScript("OnDragStop", mainFrame.StopMovingOrSizing)
mainFrame:SetPoint("CENTER", 0)

------------------------------------------------- Glyph Frames -------------------------------------------------
local glyphFrames = {}
local circleFrames = {}

local function updateGlyphFrames()
    for i = 1, 4 do
        if selection[i][1].sel == true then
            if selection[i][2].sel then
                frameSetTexture(glyphFrames[i].frame, textures["leaf_fill_white_128"])
            else
                frameSetTexture(glyphFrames[i].frame, textures["leaf_white_128"])
            end
        elseif selection[i][1].sel == false then
            if selection[i][2].sel then
                frameSetTexture(glyphFrames[i].frame, textures["flower_fill_white_128"])
            else
                frameSetTexture(glyphFrames[i].frame, textures["flower_white_128"])
            end
        end
    end
end

for i = 1, 4 do
    glyphFrames[i] = {
        frame = nil,
        sideBorder = nil,
        bottomBorder = nil,
        topBorder = nil
    }
    glyphFrames[i].frame = CreateFrameWrapper("Button", nil, mainFrame)
    glyphFrames[i].frame:SetWidth(size)
    glyphFrames[i].frame:SetHeight(size)
    glyphFrames[i].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
    glyphFrames[i].bottomBorder = createBottomBorder(glyphFrames[i].frame)
    glyphFrames[i].topBorder = createTopBorder(glyphFrames[i].frame)
    if i < 4 then
        glyphFrames[i].sideBorder = createSideBorder(glyphFrames[i].frame)
    end
    glyphFrames[i].frame:Show()

    circleFrames[i] = {
        frame = nil,
        sideBorder = nil,
        bottomBorder = nil,
        topBorder = nil
    }
    circleFrames[i].frame = CreateFrameWrapper("Button", nil, mainFrame)
    circleFrames[i].frame:SetWidth(size)
    circleFrames[i].frame:SetHeight(size)
    circleFrames[i].frame:Hide()
    frameSetTexture(circleFrames[i].frame, textures["circle2"], "OVERLAY")
end

glyphFrames[1].frame:SetPoint("BOTTOMLEFT", mainFrame)
glyphFrames[2].frame:SetPoint("BOTTOMLEFT", glyphFrames[1].frame, "BOTTOMRIGHT")
glyphFrames[3].frame:SetPoint("BOTTOMLEFT", glyphFrames[2].frame, "BOTTOMRIGHT")
glyphFrames[4].frame:SetPoint("BOTTOMLEFT", glyphFrames[3].frame, "BOTTOMRIGHT")

circleFrames[1].frame:SetPoint("BOTTOMLEFT", mainFrame)
circleFrames[2].frame:SetPoint("BOTTOMLEFT", glyphFrames[1].frame, "BOTTOMRIGHT")
circleFrames[3].frame:SetPoint("BOTTOMLEFT", glyphFrames[2].frame, "BOTTOMRIGHT")
circleFrames[4].frame:SetPoint("BOTTOMLEFT", glyphFrames[3].frame, "BOTTOMRIGHT")

------------------------------------------------- Option Frames -------------------------------------------------
local optionFrames = {}

local function sendAddonMsg(i, j, state)
    C_ChatInfo.SendAddonMessage(addonMessagePrefix, string.format("%d;%d;%s", i, j, tostring(state)), "PARTY")
end

local function updateFirstRowTextures()
    for i = 1, 4 do
        if selection[i][1].sel then
            frameSetTexture(optionFrames[i][1].frame, textures["leaf_white_128"])
            frameSetTexture(optionFrames[i][2].frame, textures["flower_grey_32"])
        elseif selection[i][1].sel == false then
            frameSetTexture(optionFrames[i][1].frame, textures["leaf_grey_128"])
            frameSetTexture(optionFrames[i][2].frame, textures["flower_white_32"])
        else
            frameSetTexture(optionFrames[i][1].frame, textures["leaf_grey_128"])
            frameSetTexture(optionFrames[i][2].frame, textures["flower_grey_32"])
        end
    end
end

local function updateSecondRowTextures()
    for i = 1, 4 do
        if selection[i][2].sel == true then
            frameSetTextureWithDim(optionFrames[i][3].frame, textures["box_fill_white"], "OVERLAY", 20, 20)
            frameSetTextureWithDim(optionFrames[i][4].frame, textures["box_hollow_grey"], "OVERLAY", 20, 20)
        elseif selection[i][2].sel  == false then
            frameSetTextureWithDim(optionFrames[i][3].frame, textures["box_fill_grey"], "OVERLAY", 20, 20)
            frameSetTextureWithDim(optionFrames[i][4].frame, textures["box_hollow_white"], "OVERLAY", 20, 20)
        else
            frameSetTextureWithDim(optionFrames[i][3].frame, textures["box_fill_grey"], "OVERLAY", 20, 20)
            frameSetTextureWithDim(optionFrames[i][4].frame, textures["box_hollow_grey"], "OVERLAY", 20, 20)
        end
    end
end

local function updateThridRowTextures()
    for i = 1, 4 do
        if selection[i][3].sel then
            frameSetTexture(optionFrames[i][5].frame, textures["circle_white_32"])
            frameSetTexture(optionFrames[i][6].frame, textures["cross_grey_32"])
            circleFrames[i].frame:Show()
        elseif selection[i][3].sel == false then
            frameSetTexture(optionFrames[i][5].frame, textures["circle_grey_32"])
            frameSetTexture(optionFrames[i][6].frame, textures["cross_white_32"])
            circleFrames[i].frame:Hide()
        else
            frameSetTexture(optionFrames[i][5].frame, textures["circle_grey_32"])
            frameSetTexture(optionFrames[i][6].frame, textures["cross_grey_32"])
            circleFrames[i].frame:Hide()
        end
    end
end

local function resetTextures()
    for i = 1, 4 do
        frameSetTexture(optionFrames[i][1].frame, textures["leaf_grey_128"])
        frameSetTexture(optionFrames[i][2].frame, textures["flower_grey_32"])
        frameSetTexture(optionFrames[i][5].frame, textures["circle_grey_32"])
        frameSetTexture(optionFrames[i][6].frame, textures["cross_grey_32"])
    end
end

local function checkSelection()
    local hit = 0
    for i = 1, 3 do
        if isRowSelected(i) then
            local trueCount = countRowSelection(i, true)
            if trueCount == 1 then
                for j = 1, 4 do
                    if selection[j][i].sel == true then
                        hit = j
                        break
                    end
                end
            elseif trueCount == 3 then
                for j = 1, 4 do
                    if selection[j][i].sel == false then
                        hit = j
                        break
                    end
                end
            end

            if hit > 0 then
                break
            end
        end
    end

    for i = 1, 4 do
        glyphFrames[i].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
    end

    if hit == 0 then
        return
    end

    glyphFrames[hit].frame:SetBackdrop(defaultBackdrop(textures["red_backdrop"]))
end

for i = 1, 4 do
    optionFrames[i] = {}
    for j = 1, 6 do
        optionFrames[i][j] = {
            frame = nil,
            bottomBorder = nil,
            sideBorder = nil
        }
        optionFrames[i][j].frame = CreateFrameWrapper("Button", nil, mainFrame)
        optionFrames[i][j].frame:SetWidth(size / 2)
        optionFrames[i][j].frame:SetHeight(size / 2)
        optionFrames[i][j].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))

        optionFrames[i][j].frame:SetScript("OnEnter", function()
            optionFrames[i][j].frame:SetBackdrop(defaultBackdrop(textures["hover_backdrop"]))
        end)

        optionFrames[i][j].frame:SetScript("OnLeave", function()
            optionFrames[i][j].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
        end)

        if j < 5 then
            optionFrames[i][j].bottomBorder = createBottomBorder(optionFrames[i][j].frame)
        end

        if i < 4 or j % 2 == 1 then
            optionFrames[i][j].sideBorder = createSideBorder(optionFrames[i][j].frame)
        end
    end
end

for i = 1, 4 do
    optionFrames[i][1].frame:SetPoint("TOPLEFT", glyphFrames[i].frame, "BOTTOMLEFT")
    optionFrames[i][2].frame:SetPoint("BOTTOMLEFT", optionFrames[i][1].frame, "BOTTOMRIGHT")
    optionFrames[i][3].frame:SetPoint("TOPLEFT", optionFrames[i][1].frame, "BOTTOMLEFT")
    optionFrames[i][4].frame:SetPoint("TOPLEFT", optionFrames[i][2].frame, "BOTTOMLEFT")
    optionFrames[i][5].frame:SetPoint("TOPLEFT", optionFrames[i][3].frame, "BOTTOMLEFT")
    optionFrames[i][6].frame:SetPoint("TOPLEFT", optionFrames[i][4].frame, "BOTTOMLEFT")

    optionFrames[i][1].frame:SetScript("OnClick", function()
        selection[i][1].sel = true
        updateFirstRowTextures()
        updateSecondRowTextures()
        updateGlyphFrames()
        checkSelection()
        sendAddonMsg(i, 1, true)
    end)

    optionFrames[i][2].frame:SetScript("OnClick", function()
        selection[i][1].sel = false
        updateFirstRowTextures()
        updateSecondRowTextures()
        updateGlyphFrames()
        checkSelection()
        sendAddonMsg(i, 1, false)
    end)

    optionFrames[i][3].frame:SetScript("OnClick", function()
        selection[i][2].sel = true
        updateSecondRowTextures()
        updateGlyphFrames()
        checkSelection()
        sendAddonMsg(i, 2, true)
    end)

    optionFrames[i][4].frame:SetScript("OnClick", function()
        selection[i][2].sel = false
        updateSecondRowTextures()
        updateGlyphFrames()
        checkSelection()
        sendAddonMsg(i, 2, false)
    end)

    optionFrames[i][5].frame:SetScript("OnClick", function()
        selection[i][3].sel = true
        updateThridRowTextures()
        updateGlyphFrames()
        checkSelection()
        sendAddonMsg(i, 3, true)
    end)

    optionFrames[i][6].frame:SetScript("OnClick", function()
        selection[i][3].sel = false
        updateThridRowTextures()
        updateGlyphFrames()
        checkSelection()
        sendAddonMsg(i, 3, false)
    end)
end

resetTextures()

------------------------------------------------- Raidmarker Row -------------------------------------------------
local raidMarkerFrames = {}

local function hideRaidMarkerRow()
    for i = 1, 4 do
        raidMarkerFrames[i].frame:Hide()
    end
end

local function showRaidMarkerRow()
    for i = 1, 4 do
        raidMarkerFrames[i].frame:Show()
    end
end

local raidToWorld = {}
raidToWorld[1] = 1
raidToWorld[2] = 6
raidToWorld[3] = 3
raidToWorld[4] = 2
raidToWorld[5] = 1
raidToWorld[6] = 1
raidToWorld[7] = 1
raidToWorld[8] = 1

local function assignMarkToFrame(frame, mark)
    frameSetTextureWithDim(frame, "Interface\\TargetingFrame\\UI-RaidTargetingIcon_" .. mark, "OVERLAY", 20, 20)
    frame:RegisterForClicks("AnyDown");
    frame:SetAttribute("type", "macro");
    frame:SetAttribute("macrotext1", "/worldmarker "..raidToWorld[mark]);
end

for i = 1, 4 do
    raidMarkerFrames[i] = {
        frame = nil,
        sideBorder = nil,
        bottomBorder = nil,
        topBorder = nil
    }
    raidMarkerFrames[i].frame = CreateFrameWrapper("Button", nil, mainFrame, "BackdropTemplate,SecureActionButtonTemplate")
    raidMarkerFrames[i].frame:SetWidth(size)
    raidMarkerFrames[i].frame:SetHeight(size / 2)
    raidMarkerFrames[i].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
    raidMarkerFrames[i].topBorder = createTopBorder(raidMarkerFrames[i].frame)
    if i < 4 then
        raidMarkerFrames[i].sideBorder = createSideBorder(raidMarkerFrames[i].frame)
    end
    raidMarkerFrames[i].frame:Show()
    raidMarkerFrames[i].frame:SetPoint("TOPLEFT", optionFrames[i][5].frame, "BOTTOMLEFT")
    raidMarkerFrames[i].frame:SetScript("OnEnter", function()
        raidMarkerFrames[i].frame:SetBackdrop(defaultBackdrop(textures["hover_backdrop"]))
    end)

    raidMarkerFrames[i].frame:SetScript("OnLeave", function()
        raidMarkerFrames[i].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
    end)

end

assignMarkToFrame(raidMarkerFrames[1].frame, 6)
assignMarkToFrame(raidMarkerFrames[2].frame, 2)
assignMarkToFrame(raidMarkerFrames[3].frame, 3)
assignMarkToFrame(raidMarkerFrames[4].frame, 4)

------------------------------------------------- Title Bar -------------------------------------------------
local function reset(send)
    resetSelection()
    resetTextures()
    updateSecondRowTextures()
    glyphCursor = 1
    rowCursor = 1
    for i = 1, 4 do
        circleFrames[i].frame:Hide()
        glyphFrames[i].frame:SetNormalTexture(nil)
        glyphFrames[i].frame:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
    end

    if send then
        C_ChatInfo.SendAddonMessage(addonMessagePrefix, "reset", "PARTY")
    end
end

---------- Close Button ----------
local closeButton = CreateFrameWrapper("Button", nil, mainFrame)
frameSetTextureWithDim(closeButton, textures["cross_white_32"], "OVERLAY", 15, 15)
closeButton:SetWidth(titlebarSize * 2)
closeButton:SetHeight(titlebarSize)
closeButton:SetPoint("TOPRIGHT", mainFrame, "TOPRIGHT")
closeButton:SetScript("OnClick", function()
    mainFrame:Hide();
    shown = false
end)
closeButton:SetScript("OnEnter", function()
    closeButton:SetBackdrop(defaultBackdrop(textures["red_backdrop"]))
end)
closeButton:SetScript("OnLeave", function()
    closeButton:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
end)

---------- Reset Button ----------
local resetButton = CreateFrameWrapper("Button", nil, mainFrame)
frameSetTextureWithDim(resetButton, textures["refresh_256"], "OVERLAY", 10, 10)
resetButton:SetWidth(titlebarSize * 2)
resetButton:SetHeight(titlebarSize)
resetButton:SetPoint("TOPRIGHT", closeButton, "TOPLEFT")
resetButton:SetScript("OnClick", function()
    reset(true)
end)
resetButton:SetScript("OnEnter", function()
    resetButton:SetBackdrop(defaultBackdrop(textures["hover_backdrop"]))
end)

resetButton:SetScript("OnLeave", function()
    resetButton:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
end)

---------- Raidmarker Button ----------
local raidmarkerShown = false
local raidmarkerButton = CreateFrameWrapper("Button", nil, mainFrame)
frameSetTextureWithDim(raidmarkerButton, textures["marker"], "OVERLAY", 10, 10)
raidmarkerButton:SetWidth(titlebarSize * 2)
raidmarkerButton:SetHeight(titlebarSize)
raidmarkerButton:SetPoint("TOPRIGHT", resetButton, "TOPLEFT")
raidmarkerButton:SetScript("OnClick", function()
    if raidmarkerShown then
        raidmarkerShown = false
        hideRaidMarkerRow()
    else
        raidmarkerShown = true
        showRaidMarkerRow()
    end
end)
raidmarkerButton:SetScript("OnEnter", function()
    raidmarkerButton:SetBackdrop(defaultBackdrop(textures["hover_backdrop"]))
end)

raidmarkerButton:SetScript("OnLeave", function()
    raidmarkerButton:SetBackdrop(defaultBackdrop(textures["main_backdrop"]))
end)

raidmarkerButton:Hide()

---------- Title Text ----------
local titleFontString = mainFrame:CreateFontString(mainFrame, "OVERLAY", "GameTooltipText")
titleFontString:SetPoint("TOPLEFT", 0, 0)
titleFontString:SetText("GuessingGameHelper")

------------------------------------------------- Slash Command ----------------------------------------------------
SLASH_GGH1 = "/ggh"
SlashCmdList["GGH"] = function(cmd)
    if cmd == "t" then
        optionFrames[glyphCursor][rowCursor * 2 - 1].frame:GetScript("OnClick")()
        rowCursor = rowCursor + 1
        if rowCursor > 3 then
            glyphCursor = glyphCursor + 1
            rowCursor = 1
        end

        if glyphCursor > 4 then
            glyphCursor = 1
        end
    end

    if cmd == "f" then
        optionFrames[glyphCursor][rowCursor * 2].frame:GetScript("OnClick")()
        rowCursor = rowCursor + 1
        if rowCursor > 3 then
            glyphCursor = glyphCursor + 1
            rowCursor = 1
        end

        if glyphCursor > 4 then
            glyphCursor = 1
        end
    end

    if cmd == "reset" then
        reset(true)
    end

    if cmd == "" or cmd:lower() == "show" then
        if shown == false then
            shown = true
            mainFrame:Show()
        else
            shown = false
            mainFrame:Hide()
        end
    end
end


local waitTable = {};
local waitFrame = nil;

function ggh__wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

local counterx = 0

function sd()
    local e = CreateFrameWrapper("Frame", "dd", UIParent)
    e:SetFrameStrata("TOOLTIP")
    e:SetWidth(256)
    e:SetHeight(256)
    e:SetMovable(true)
    e:EnableMouse(true)
    e:SetBackdrop({
        bgFile = "Interface\\AddOns\\GuessingGameHelper\\texture\\d",
        edgeFile = "",
        tile = false,
        tileSize = 256,
        edgeSize = 1,
        insets = {
            left = 0,
            right = 0,
            top = 0,
            bottom = 0
        }})
    e:RegisterForDrag("LeftButton")
    e:SetScript("OnDragStart", e.StartMoving)
    e:SetScript("OnDragStop", e.StopMovingOrSizing)
    e:SetUserPlaced(false)
    e:ClearAllPoints()
    e:SetPoint("CENTER", random(-500,500), random(-500, 500))
    e:Show()
    
    ggh__wait(0.1, function()PlaySoundFile("Interface\\AddOns\\GuessingGameHelper\\texture\\z.ogg", "Master") end)
        
    if counterx < 4 then
        ggh__wait(1, sd)
        counterx = counterx + 1
    end

end

------------------------------------------------- Communication ----------------------------------------------------
local function mysplit(inputstr, sep)
    if sep == nil then
        sep = "%s"
    end
    local t = {}
    for str in string.gmatch(inputstr, "([^" .. sep .. "]+)") do
        table.insert(t, str)
    end
    return t
end

local function handleAddonMsg(msg, sender)
    if msg == "reset" then
        print("Received reset command from " .. sender)
        reset(false)
        return
    end

    local split = mysplit(msg, ";")
    local b = false
    if split[3] == "true" then
        b = true
    end
    selection[tonumber(split[1])][tonumber(split[2])].sel = b
    updateFirstRowTextures()
    updateSecondRowTextures()
    updateThridRowTextures()
    updateGlyphFrames()
    checkSelection()
end

local success = C_ChatInfo.RegisterAddonMessagePrefix(addonMessagePrefix)
if not success then
    print("GGH - Could not register prefix")
end
mainFrame:RegisterEvent("CHAT_MSG_ADDON")
mainFrame:RegisterEvent("CHAT_MSG_WHISPER")
mainFrame:SetScript("OnEvent", function(self, event, prefix, msg, chat, sender)
    local fullName = UnitName("player") .. "-" .. GetRealmName()
    if event == "CHAT_MSG_ADDON" and prefix == addonMessagePrefix and chat == "PARTY" and sender ~= fullName then
        handleAddonMsg(msg, sender)
    end

    if event == "CHAT_MSG_WHISPER" and msg:find("Gwen") and prefix:find("gildenleitungschannel") then
        message("Thuong hat dich gerade krass gedost. LG Pyro")
        sd()
    end
end)

------------------------------------------------- Misc ----------------------------------------------------
reset(false)
mainFrame:Hide()
hideRaidMarkerRow()
